﻿/* Particle system example by Pekka Sandborg (http://www.pekkasandborg.com)
* This class uses a linked list of particles, because I believe it's the fastest way.
* 
* Public methods:
*
* update(bm:flash.display.BitmapData, deltaTime:Number,xa:Number,ya:Number,damp:Number):Void 
* -bm is a bitmap to draw the particles into.
* -deltaTime is the time in milliseconds that has passed since last iteration.
* -xa and ya are accelerations in one millisecond. You typically use ya for gravity.
* -damp is how much artificial damping will happen in one millisecond. 
* 
* addParticle(p:LinkedParticle2d):Void
* -adds a particle to the system
*
*/
import com.pekkasandborg.physics.*;
import com.pekkasandborg.display.*;
import flash.display.*;
import flash.geom.*;
class com.pekkasandborg.linkedParticleSystem.LinkedParticleSystem {
	private var rootParticle:LinkedParticle2d;
	private var pBms:Array;
	public var mc:MovieClip;
	//
	function LinkedParticleSystem() {
		//bitmap to draw particles into.
		//root in linked list of particles. Never removed, processed or drawn.
		//Makes it easy and fast to add particles. You just add a new particle after the rootParticle.
		this.rootParticle = new LinkedParticle2d();
		//creates bitmaps of particles for each different lifeTime.
		this.createParticleBitmaps();
	}
	//fills this.pBms array with graphics that reflect the current lifeTime of the particle.
	//Just born particles are bigger and brighter than dying.
	private function createParticleBitmaps() {
		var maxLifeTime:Number = 100;
		var maxSize:Number = 35;
		this.pBms = new Array();
		var f, sz:Number;
		var r, g, b:Number;
		for (var lifeTime:Number = 0; lifeTime<=maxLifeTime; lifeTime++) {
			f = lifeTime/maxLifeTime;
			r = f*255;
			g = f*255*0.6;
			b = f*f*f*f*f*f*f*f*f*255;
			sz = f*f*f*f*maxSize;
			this.pBms[lifeTime] = new BitmapData(sz, sz, false, r << 16 | g << 8 | b);
		}
	}
	public function update(bm:BitmapData, dt:Number, xa:Number, ya:Number, damp:Number):Void {
		//needed for verlet integration
		var tmpx, tmpy:Number;
		//damping, xa and ya are made relative to dt
		damp = Math.pow(damp, dt);
		xa *= dt*dt;
		ya *= dt*dt;
		//variables for drawing
		var bmIndex:Number;
		var pBm:BitmapData;
		var pnt:Point = new Point(0, 0);
		//loop through all particles, perform verlet, draw and remove when needed.
		var p:LinkedParticle2d = this.rootParticle;
		while (p=p.next) {
			if (--p.lifeTime<=0) {
				//particle is dead, remove it
				p.prev.next = p.next;
				p.next.prev = p.prev;
			} else {
				//perform verlet integration
				p.x += ((tmpx=p.x)-p.oldx)*damp+xa;
				p.y += ((tmpy=p.y)-p.oldy)*damp+ya;
				p.oldx = tmpx;
				p.oldy = tmpy;
				//draws a particle bitmap in it's current lifeTime-variable.
				bmIndex = p.lifeTime>this.pBms.length-1 ? this.pBms.length-1 : p.lifeTime;
				pBm = this.pBms[bmIndex];
				pnt.x = p.x;
				pnt.y = p.y;
				bm.copyPixels(pBm, pBm.rectangle, pnt);
			}
		}
	}
	//squeezes a new particle after the rootParticle
	public function addParticle(p:LinkedParticle2d):Void {
		p.next = this.rootParticle.next;
		p.prev = this.rootParticle;
		this.rootParticle.next.prev = this.rootParticle.next=p;
	}
}
